Imports PCBI
Imports PCBI.Automation
Imports PCBI.MathUtils
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Diagnostics
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms

Namespace Demo10VB
    Public Class Form1
        Inherits Form

        Public Class ResultForHistogram
            Public LayerName As String = ""

            Public CountLines As Integer = 0

            Public CountArcs As Integer = 0

            Public CountSurfaces As Integer = 0

            Public CountText As Integer = 0

            Public CountPads As Integer = 0

            Public SmallesElementArea As Double = 1.7976931348623157E+308

            Public Function ToString() As String
                Return String.Concat(Me.LayerName, ": L ", Me.CountLines, ", A ", Me.CountArcs, ", S ", Me.CountSurfaces, ", T ", Me.CountText, ", P ", Me.CountPads, " - smallest elemts size: ", Me.SmallesElementArea.ToString(), " mils²")
            End Function
        End Class

        Private components As IContainer = Nothing

        Private WithEvents button1 As Button

        Private mainWindowPCBI As IPCBIWindow

        Protected Sub Dispose(ByVal disposing As Boolean)
            If disposing AndAlso Me.components IsNot Nothing Then
                Me.components.Dispose()
            End If
            MyBase.Dispose(disposing)
        End Sub

        Private Sub InitializeComponent()
            Me.button1 = New Button()
            MyBase.SuspendLayout()
            Me.button1.Location = New Point(12, 12)
            Me.button1.Name = "button1"
            Me.button1.Size = New Size(260, 73)
            Me.button1.TabIndex = 0
            Me.button1.Text = "Demo 10: Create histogram for all signal layers"
            Me.button1.UseVisualStyleBackColor = True
            MyBase.AutoScaleDimensions = New SizeF(6.0F, 13.0F)
            MyBase.AutoScaleMode = AutoScaleMode.Font
            MyBase.ClientSize = New Size(284, 99)
            MyBase.Controls.Add(Me.button1)
            MyBase.Name = "Form1"
            Me.Text = "Demo 10"
            MyBase.ResumeLayout(False)
        End Sub

        Public Sub New()
            Me.InitializeComponent()
            IAutomation.IAutomationInit()
            Me.mainWindowPCBI = IAutomation.CreateNewPCBIWindow(True)
        End Sub

        Private Sub buttonExportImage_Click(ByVal sender As Object, ByVal e As EventArgs) Handles button1.Click
            Dim [step] As IStep = Me.mainWindowPCBI.GetCurrentStep()
            If [step] Is Nothing Then
                MessageBox.Show("Plese load a job first to use this demo.", "No Job Loaded!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk)
            Else
                Dim matrix As IMatrix = Me.mainWindowPCBI.GetMatrix()
                Dim ResultOverviewList As List(Of Form1.ResultForHistogram) = New List(Of Form1.ResultForHistogram)()
                For Each layerName As String In matrix.GetAllSignalLayerNames()
                    Dim relevantLayer As ILayer = [step].GetLayer(layerName)
                    If relevantLayer IsNot Nothing Then
                        Dim layerObjectList As List(Of IObject) = relevantLayer.GetAllLayerObjects()
                        If layerObjectList.Count = 0 Then
                            Return
                        End If
                        Dim smallestArea As Double = 1.7976931348623157E+308
                        Dim smallesObject As IODBObject = Nothing
                        Dim layerHistogram As Form1.ResultForHistogram = New Form1.ResultForHistogram()
                        layerHistogram.LayerName = layerName
                        For Each relevantObject As IODBObject In relevantLayer.GetAllLayerObjects()
                            If relevantObject.Type = IObjectType.Pad Then
                                layerHistogram.CountPads += 1
                                Dim pad As IPadSpecificsD = CType(relevantObject.GetSpecificsD(), IPadSpecificsD)
                                If pad.Type = Symbol_Type.rect Then
                                    Dim boundsRelevanObject As RectangleD = relevantObject.GetBoundsD()
                                    Dim areaOfPad As Double = boundsRelevanObject.Width * boundsRelevanObject.Height
                                    If areaOfPad < smallestArea Then
                                        smallesObject = relevantObject
                                        smallestArea = areaOfPad
                                    End If
                                Else
                                    If pad.Type = Symbol_Type.r Then
                                        Dim areaOfPad As Double = Math.Pow(pad.Diameter / 2.0, 2.0) * 3.1415926535897931
                                        If areaOfPad < smallestArea Then
                                            smallesObject = relevantObject
                                            smallestArea = areaOfPad
                                        End If
                                    Else
                                        Dim areaOfPad As Double = CDec(relevantObject.CalculateArea(True))
                                        If areaOfPad < smallestArea Then
                                            smallesObject = relevantObject
                                            smallestArea = areaOfPad
                                        End If
                                    End If
                                End If
                            Else
                                If relevantObject.Type = IObjectType.Arc Then
                                    layerHistogram.CountArcs += 1
                                Else
                                    If relevantObject.Type = IObjectType.Line Then
                                        layerHistogram.CountLines += 1
                                    Else
                                        If relevantObject.Type = IObjectType.Surface Then
                                            layerHistogram.CountSurfaces += 1
                                        Else
                                            If relevantObject.Type = IObjectType.Text Then
                                                layerHistogram.CountText += 1
                                            End If
                                        End If
                                    End If
                                End If
                                Dim areaOfPad As Double = CDec(relevantObject.CalculateArea(True))
                                If areaOfPad < smallestArea Then
                                    smallesObject = relevantObject
                                    smallestArea = areaOfPad
                                End If
                            End If
                        Next

                        If smallesObject IsNot Nothing Then
                            smallesObject.[Select](True)
                            layerHistogram.SmallesElementArea = smallestArea
                        End If
                        ResultOverviewList.Add(layerHistogram)
                    End If
                Next
                Dim errorLog As String = IAutomation.GetErrorLog()
                If errorLog.Length > 0 Then
                    Debug.WriteLine(errorLog)
                End If
                If ResultOverviewList.Count > 0 Then
                    Dim sb As StringBuilder = New StringBuilder()
                    For Each resultLayerHistogram As Form1.ResultForHistogram In ResultOverviewList
                        sb.AppendLine(resultLayerHistogram.ToString())
                    Next
                    MessageBox.Show("Overview all signal layers " + Environment.NewLine + sb.ToString(), "Layer Histogram", MessageBoxButtons.OK, MessageBoxIcon.Asterisk)
                Else
                    MessageBox.Show("No results found, this demo works only for signal layers!", "No Layer Histogram", MessageBoxButtons.OK, MessageBoxIcon.Asterisk)
                End If
            End If
        End Sub
    End Class
End Namespace
